var myService = new function()
{
    var undefined;
    var _config = {};
    _config.itemsPerPage    = 12;
    _config.pagesAround     = 5;
    
    _config.section         = 'content';

    _config.minAge          = 1000*10;      // 10 seconds sliding validity of the _folderCache
    _config.maxAge          = 1000*5*60;    // 5 minutes max validity of the _folderCache
    _config.thumbsSize      = 80;
    _config.thumbsQuality   = .6;
    _config.previewSize     = 480*1.5;
    _config.previewQuality  = .85;

    var _activityStream     = new ActivityStream();

    var _foldersCache       = {};
    var _orientation        = JSON.parse( widget.preferenceForKey( '_orientation' )||'{}' );
    var _angles             = 
    {
        'n':0,'e':1,'s':2,'w':3,
        'N':0,'E':1,'S':2,'W':3,
        0:'n',1:'e',2:'s',3:'w'
    };

    /*
     *  alias of getContentType
     */
    var _getContentType = opera.io.webserver.getContentType;

    function init()
    {
        Yusef.plugins.ui.setStyleSheet( "static/style.css" );
    }
    
    /*
     *  Is the file an image ?
     *  @private
     */
    function _isImage( file )
    {
        return _getContentType( file.path ).slice(0,6)==='image/';
    }

    /*
     *  Yusef.addSectionListener '_index'
     */
    Yusef.addSectionListener
    (
        '_index',
        function( connection )
        {
            connection.response.setStatusCode( '302', 'Found' );
            connection.response.setResponseHeader( 'Location', 'content/' );
            connection.response.close();
        },
        {
            'ui':true
        }
    );


    /*
     *  Get the folder cache.
     *  @private
     */
    function _getFolderCache( folder )
    {
        if( folder.exists==true && folder.isDirectory==true )
        {
            //  build the _foldersCache
            var now = new Date().getTime();
            var mock = _foldersCache[ folder.path ];
            if( !mock || mock.updated<now-_config.maxAge )
            {
                var mock = {'path':folder.path.slice(Yusef.mountpoints.shared.path.length)+'/','updated':now,delta:0,'pages':0,'folders':[],'images':[]};

                folder.refresh();

                var l=Yusef.mountpoints.shared.path.length;
                for( var i=0,f=null; f=folder[i]; i++ )
                {
                    if( f.isHidden || f.isArchive){ continue; }
                    if( f.isDirectory && !f.fileSize )
                    {
                        mock.folders.push( _getFileInfo(f) );
                    }
                    else if( _isImage( f ) )
                    {
                        mock.images.push( _getFileInfo(f) );
                    }
                }

                //  sort
                function _sort( a,b ){ return a.path>b.path?1:-1; };
                mock.folders.sort( _sort );
                mock.images.sort( _sort );

                //  set page and index
                var j = mock.folders.length;
                var i = mock.images.length;
                mock.pages  = (j+i)?1+(j+i)/_config.itemsPerPage>>0:0;
                while( i-- )
                {
                    mock.images[i].index    = i+1;
                    mock.images[i].page     = 1+(j+i)/_config.itemsPerPage>>0;
                }
                while( j-- )
                {
                    mock.folders[j].page    = 1+j/_config.itemsPerPage>>0;
                }
            }

            //  sliding validity
            if( mock.updated>now-_config.minAge )
            {
                mock.updated = now;
            }
            mock.delta = new Date()-now;

            _foldersCache[ folder.path ] = mock;

            //  return a copy of the _foldersCache
            return _copyObject( mock );
        }
    }


    /*
     *  Get file HTMLName, path, size, ...
     *  @private
     */
    function _getFileInfo( file )
    {
        var tmp = {'path':file.path.slice(Yusef.mountpoints.shared.path.length)+'/'};

        //  HTMLName
        try
        {
            tmp.HTMLName    = decodeURI( file.name );
        }
        catch( e )
        {
            tmp.HTMLName    = unescape( file.name );
        }

        //  file|directory
        if( !file.isDirectory && file.fileSize )
        {
            tmp.size    = file.fileSize;
            tmp.path    = tmp.path.slice( 0,-1 );
            tmp.hash    = _hash( file.path );
        }
        else
        {
            tmp.isDirectory = true;
        }
        return tmp;
    }


    /*
     *  Get the PhotoSharing data about the file/folder.
     *  @private
     */
    function _getData( connection, file, pageIndex )
    {
        var isImage     = file.exists&&_isImage( file ),
            folder      = !file.exists||isImage?file.parent:file,
            folderCache = _getFolderCache( folder ),
            data        = { 'albumPath':folderCache.path,'breadcrumb':{},'pageIndex':pageIndex||1,'page':[] };
        data.isOwner = connection.isOwner;
        data.notFound   = file.exists==false;
        //  breadcrumb
        if( folder.parent.exists )
        {
            data.breadcrumb.HTMLName    = _getFileInfo( folder ).HTMLName;
            var tmp = _getFileInfo( folder.parent );
            data.breadcrumb.parentPath  = opera.io.webserver.currentServicePath+'content'+ tmp.path;
            data.breadcrumb.parentName  = tmp.path!=='/'?tmp.HTMLName:opera.io.webserver.currentServiceName;
        }
        else
        {
            data.breadcrumb.HTMLName  = opera.io.webserver.currentServiceName;
        }


        if( isImage )
        {
            //  look for the image and pageIndex
            var tmp = _getFileInfo( file );
            for( var i=0,f; f=folderCache.images[i]; i++ )
            {
                if( f.HTMLName === tmp.HTMLName )
                {
                    data.currentImage   = f;
                    data.pageIndex      = f.page;
                    break;
                }
            }
        }
        else
        {
            //  pick the first image ( on the page )
            data.pageIndex = Math.min( Math.max( data.pageIndex, 1 ), folderCache.pages );
            data.currentImage   = folderCache.images[ (data.pageIndex-1)*_config.itemsPerPage-folderCache.folders.length ]||folderCache.images[0];
        }


        if( data.currentImage )
        {
            var t = _orientation[ data.currentImage.hash ]||0;
            data.currentImage.cw    = _angles[ t+1&3 ];
            data.currentImage.ccw   = _angles[ t+3&3 ];

            // first, last, previous, next image
            data.isFirstImage   = data.currentImage.index==1;
            data.isLastImage    = data.currentImage.index==folderCache.images.length;
            if( !data.isFirstImage )
            {
                data.previousImage  = folderCache.images[data.currentImage.index-2];
            }
            if( !data.isLastImage )
            {
                data.nextImage  = folderCache.images[data.currentImage.index];
            }
        }
        data.hasImage       = folderCache.images.length>0;
        data.hasImages      = folderCache.images.length>1;
        data.imagesCount    = folderCache.images.length;
        data.hasFolders     = folderCache.folders.length>0;
        data.foldersCount   = folderCache.folders.length;

        var jsObject        = {};
        jsObject.path       = opera.io.webserver.currentServicePath+_config.section+folderCache.path;
        jsObject.index      = data.currentImage?data.currentImage.index-1:0;
        jsObject.images     = [];
        folderCache.images.map( function( v ){ jsObject.images.push( v.path.slice( v.path.lastIndexOf('/')+1 ) ); } );
        data.jsObject       = JSON.stringify( jsObject );

        if( data.pageIndex )
        {
            var f,
                end = data.pageIndex*_config.itemsPerPage,
                i   = end-_config.itemsPerPage;

            while( i<end && (f=folderCache.folders[i])!==undefined )
            {
                data.page.push( f );
                i++;
            }
            while( i<end && (f=folderCache.images[i-folderCache.folders.length])!==undefined )
            {
                f.orientation   = _angles[ _orientation[ f.hash ]||0 ];
                data.page.push( f );
                i++;
            }

            data.pages = [];
            for( var i=Math.max( 1, data.pageIndex-_config.pagesAround ); i<=Math.min( data.pageIndex+_config.pagesAround, folderCache.pages ); i++ )
            {
                data.pages.push( { 'name':i,'index':i,'isCurrent':i==data.pageIndex } );
                if( i==data.pageIndex-1 )data.pages.prev = data.pages[ data.pages.length-1 ];
                else if( i==data.pageIndex+1 )data.pages.next = data.pages[ data.pages.length-1 ];
            }
        }

        return data;
    }


    /*
     *  Emit nice page.
     *  @private
     */
    function _emitPage( connection, serviceData )
    {
        var output      = {};
        var yusefData   = Yusef.getData( connection );
        var template    = new Markuper( 'templates/index.html', yusefData );
        var sidebar     = new Markuper( 'templates/sidebar.html', yusefData );

        if( template )
        {
            template.parse( serviceData );
            output.content  = template.select( 'content > *' );
            if( sidebar )
            {
                output.sideBar = sidebar.select();
            }
        }

        //  output data + style + tabs
        //output.styles = [{href:'style.css'}];
        var servicePath = opera.io.webserver.currentServicePath;
        output.tabs =
        [
            {
                "title"  : opera.io.webserver.currentServiceName,
                "section": "content",
                "url"    : opera.io.webserver.currentServicePath,
                "class"  : 'active'
            },
            {
                "title"  : _("Activity Feed"), 
                "url"    : opera.io.webserver.currentServicePath + "activity/"
            }
        ];
        
        if( connection.isOwner )
        {
            output.tabs.push
            ({
                "title"  : _("Help"), 
                "url"    : opera.io.webserver.currentServicePath + "help/"
            });
        }
        
        output.data = serviceData;
        output.plugins =
        {
            sharedFolderUI  : true
        };

        return output;
    }


    /*
     *  Set the orientation of an image.
     *  @private
     */
    function _setOrientation( file, newAngle )
    {
        if( file.exists && _isImage( file ) )
        {
            var hash = _hash( file.path );
            var now  = _orientation[hash]||0;
            if( now!=newAngle )
            {
                if( newAngle )
                {
                    _orientation[hash] = newAngle;
                }
                else
                {
                    delete _orientation[hash];
                }
                widget.setPreferenceForKey( JSON.stringify( _orientation ), '_orientation' );
            }
            return _orientation[hash]||0;
        }
        return 0;
    }

    function _showOriginalFileNotification(connection, file, mediaLink)
    {
        if( connection.isOwner ) { return; }
        
        var path = file.path.slice(Yusef.mountpoints.shared.path.length);
        path = path.slice(1);
        widget.showNotification(_("{photo} was viewed in original size.", {photo: unescape(path)}), function(){
            //Override the default behaviour and open the mediaLink.
        });
    }

    //  */*/view    -> album/image/view
    Yusef.addRoute
    (
        /content\/(.*\/)?([^\/]+)\/view(\/([nesw]))?/,
        function( connection, matches )
        {
            matches[0] = (matches[0]||'');
            var data    = {};
            var file    = matches[1];
            var folder  = matches[0]==matches[1]+'/'?'/':'/'+matches[0];

            file = Yusef.mountpoints.shared.resolve('/shared/'+ folder+file );


            if( file.exists )
            {
                if( connection.isOwner && matches[3] )
                {
                    var angle = _angles[ matches[3] ];
                    _setOrientation( file, angle );
                }

                var folder = file.parent;
                if( folder.exists && folder.isDirectory && !folder.fileSize )
                {
                    data = _getData( connection, file );
                }

                return _emitPage( connection, data );
            }

            return Yusef.emitErrorPage( connection, 'file not found' );
        },
        {
            'ui':true
        }
    );


    //  */  -> album?page=n
    Yusef.addRoute
    (
        /content\/(.*\/)?/,
        function( connection, matches )
        {
            matches[0] = (matches[0]||'');
            var data    = {};
            var page    = parseInt( (connection.request.getItem('page','GET')||[1])[0]||0, 10 );
            var folder  = Yusef.mountpoints.shared.resolve( '/shared/'+ matches[0] );

            if( folder.exists && folder.isDirectory && !folder.fileSize )
            {
                data =_getData( connection, folder, page );
                return _emitPage( connection, data );
            }

            return Yusef.emitErrorPage( connection, 'file not found' );
        },
        {
            'ui':true
        }
    );


    //  */*/preview|thumbnail|original|download -> album/image/preview|thumbnail|original|download
    Yusef.addRoute
    (
        /content\/(.*\/)?([^\/]+)\/(original|download|(preview|thumbnail)(\/([nesw]))?)/,
        function( connection, matches )
        {
            matches[0] = (matches[0]||'');
            var action  = (matches[3]||matches[2]).toLowerCase();
            var file    = matches[1];
            var folder  = matches[0]==matches[1]+'/'?'':matches[0];


            file = Yusef.mountpoints.shared.resolve('/shared/'+ folder+file );
            if( !file.exists )
            {
                return Yusef.emitErrorPage( connection, 'file not found' );
            }

            connection.pending  = true;
            connection.response.setResponseHeader( 'Content-Disposition', 'inline; filename='+ file.name );


            //  -> original
            if( action=='original' )
            {
                //  publish activity
                var mediaLink = 'http://'+ connection.request.headers.Host[0]+ opera.io.webserver.currentServicePath + 'content/'+ file.name + '/view';
                _activityStream.publish( 'Image '+ file.name +' was viewed', ActivityVerbType.View, new ActivityObjectModel( ActivityObjectType.Photo, file.name, mediaLink ), null );

                _showOriginalFileNotification(connection, file, mediaLink);
                return Yusef.serveFile( connection, file );
            }

            //  -> download
            if( action=='download' )
            {
                return Yusef.serveFile( connection, file, 'application/octet-stream' );
            }

            //  -> preview|thumbnail
            var args = {};
            args.crop       = false;
            args.maxWidth   = _config.previewSize;
            args.maxHeight  = _config.previewSize;
            args.quality    = _config.previewQuality;
            if( connection.isOwner && matches[5] )
            {
                var angle   = _angles[ matches[5] ];
                _setOrientation( file, angle );
            }
            args.angle      = _orientation[ _hash( file.path ) ]||0;
            args.hash       = _hash( ([file.path,args.maxWidth,args.maxHeight,args.crop,args.quality]).join('\n') )+args.angle;


            function _serveResizedImage( data )
            {
                if( data.file && data.file.exists )
                {
                    Yusef.serveFile( connection, data.file );
                }
                else
                {
                    Yusef.emitErrorPage( connection, 'failed to resize an image' );
                }
            }

            return Yusef.plugins.graphics.resizeImage
            (
                file,
                args,
                action=='preview'?_serveResizedImage:function _image_thumbnail( data )
                {
                    args.crop       = true;
                    args.maxWidth   = _config.thumbsSize;
                    args.maxHeight  = _config.thumbsSize;
                    args.quality    = _config.thumbsQuality;
                    args.hash       = _hash( ([file.path,args.maxWidth,args.maxHeight,args.crop,args.quality]).join('\n') )+args.angle;
                    args.srcCanvas  = data.canvas;
                    args.srcFile    = data.file;
                    delete args.angle;  //  no rotation on the 'canvas'

                    Yusef.plugins.graphics.resizeImage
                    (
                        file,
                        args,
                        _serveResizedImage
                    );
                }
            );
        },
        {
            'ui':false
        }
    );
    
    init();
}
